  
;**********************************************************************
;  Filename: AS4B27.asm        File Version: 4B27
;                
;  Author: Pavel Janko  jankop@volny.cz                             
;**********************************************************************
;    Files required: <P12F675.INC>                                                 
;                                                                     
;                                                                     
;**********************************************************************
;    Notes:
;  
;
;
;
; 
;
;                                                           
;**********************************************************************
	list      p=12F675  		; list directive to define processor
	#include <P12F675.INC>  ; processor specific variable definitions
	errorlevel -302
	__CONFIG  _CP_OFF &_CPD_OFF & _WDT_ON & _MCLRE_ON &_BODEN_OFF &_PWRTE_ON &_INTRC_OSC_NOCLKOUT 
	__idlocs 0x4B27 			; Uklada verzi do ctyr ID byte (pouze 4 spodni bity)
					    		   ; Datum je kodovano hexadecimalne xYxMdDdD
	#define	  	RAM_Start 		0x20 
	#define		counter_over	0x00	; counter flag for sampling increment to 0x0000
	#define		delay_over		0x01	; counter flag for delay decrement to 0xFF
	#define		Q_OUT				GPIO0 ; Vystup
	#define		NOT_Q_OUT		GPIO1 ; Negovany vystup
	#define 		NC1				GPIO2 ; REZERVA
	#define		NC2				GPIO4	; REZERVA - Krystal, jako vystup LOW
	#define		NC3				GPIO5	; REZERVA - Krystal, jako vystup LOW	
 ;--------- CONSTANTS DEFINITIONS---------------------------------------
	#define const2hodLO		0x5A	; konstanta pro 2hod 
	#define const2hodHI		0xCA	; konstanta pro 2hod 
	#define const10sLO		0xED	; konstanta pro 10s
	#define const10sHI		0xFF	; konstanta pro 10s
	
	;--------- VARIABLE DEFINITIONS ---------------------------------------
	CBLOCK 	RAM_Start
w_temp        	; variable used for context saving 
status_temp   	; variable used for context saving
counterLO		; citac sample x 0,5 sekund
counterHI		; 				"
ucrLO				; universalni registr LO
ucrHI				; universalni registr HI
flag				; registr priznaku
mask				; pomocny registr				
	ENDC


;----------------------------------------------------------------------
		ORG     0x000          	; Processor reset vector
reset
		goto main					; Go to beginning of program

		ORG     0x004          	; interrupt vector location

;----------- Obsluha preruseni vcetne ulozeni registru ------------
		movwf w_temp         	; save off current W register contents
		movf STATUS,W        	; move status register into W register
		movwf status_temp     	; save off contents of STATUS register
		bcf STATUS, RP0 			; Restore Bank 0
		btfsc PIR1,TMR1IF			; test preruseni od timeru casu	
		goto timer1_INT			; preruseni od timeru casu	
		goto reset					; osetreni poruchy 
;----------------------------------------------------------------------
; 				Obsluha preruseni od TMR1  
; Kazdym pruchodem je inkrementovan par registru counterLO/HI,
; pokud neni aktivni bit flag<counter_over>. Ten je nastaven, kdyz je 
; counterLO/HI incrementovan na 0x0000. Muze byt rizen i z vnejsku.
;----------------------------------------------------------------------
timer1_INT
		btfsc flag,counter_over	; preteceni registru -> 
		goto ctstop					; neincrementovat
		incf counterLO,F			;		"
		btfsc STATUS,Z				;		"
		incf counterHI,F			;		"
		btfsc STATUS,Z
		bsf flag,counter_over	; timer pro flash Roll Over
ctstop	
      bcf PIR1,TMR1IF			; mazani priznaku preruseni TMR1  
		movf status_temp,W		; retrieve copy of STATUS register
		movwf STATUS				; restore pre-isr STATUS register contents
		swapf w_temp,F
		swapf w_temp,W				; restore pre-isr W register contents
		retfie						; return from interrupt
;---------------------------------------------------------------------
; Start procesoru , konfigurace
;---------------------------------------------------------------------
main	
		clrwdt
		bcf INTCON,GIE			; zakaz vsech preruseni
		call reinit_CPU		; odstaveni periferii a procesu
      bsf STATUS, RP0 		; Restore Bank 1
		call 0x3FF 				; Get the cal value
		movwf OSCCAL 			; Calibrate
		bcf STATUS,RP0 		; Bank 0
		call set_IO				; nastaveni portu
		call set_TMR1			; nastaveni casovace TMR1
		bsf INTCON,PEIE		; Enables all unmasked perif. IRQ
		bsf INTCON,GIE			; Enables all unmasked interrupts
;---------------------------------------------------------------
;                 Hlavni smycka
;---------------------------------------------------------------
	
wait01
		bcf mask,NOT_Q_OUT			
		bsf mask,Q_OUT
		bcf mask,NC1
		bcf mask,NC2
		movf mask,W
		movwf GPIO					; nastaveni aktivniho vystupu
		bsf flag,counter_over	; stop citace 
		movlw const10sLO			; nastaveni konstant timeru
		movwf counterLO			;		"
		movlw const10sHI			; nastaveni konstant timeru
	   movwf counterHI			;		"
		bcf flag,counter_over	; start citace 
wait02
		clrwdt
		btfss flag,counter_over ; Test uplynut 10s
		goto wait02
		bsf mask,NOT_Q_OUT		 
		bcf mask,Q_OUT
		bcf mask,NC1
		bcf mask,NC2
		movf mask,W
		movwf GPIO					; nastaveni neaktivniho vystupu
		bsf flag,counter_over	; stop citace 
		movlw const2hodLO			; nastaveni konstant timeru
		movwf counterLO			;		"
		movlw const2hodHI			; nastaveni konstant timeru
	   movwf counterHI			;		"
		bcf flag,counter_over	; start citace 
wait03
		clrwdt
		btfss flag,counter_over ; Test uplynut 2 hod
		goto wait03
		goto wait01
;-----------------------------------------------------------------------
; Zastaveni cinnosti periferii a IRQ  reinicializace 
;-----------------------------------------------------------------------
reinit_CPU
		bcf STATUS, RP0 		; Restore Bank 0
		clrf INTCON				; zakaz vsech IRQ
		clrf T1CON 				; Stop Timer1,LP oscillator is off, Internal clock(F OSC /4)
		clrf ADCON0				; vypnuti ADC - snizeni spotreby
		movlw 0x07				; odpojeni komparatoru CMP
		movwf CMCON				;       "
		bsf STATUS,RP0			; Bank 1
		clrf PIE1				; zakaz vsech IRQ periferii
		clrwdt
		clrf OPTION_REG		; TMR0 internal clock source,prescaler to the TIMER0
		clrf VRCON				; vypnuti reference - snizeni spotreby
		clrf ANSEL				; odpojeni ADC
		bcf STATUS, RP0 		; Restore Bank 0
		clrf INTCON				; mazani priznaku IRQ
		clrf PIR1				; mazani priznaku IRQ periferii
		return
;----------------------------------------------------------------------
; Nastaven vstupu a vystupu.
;----------------------------------------------------------------------
set_IO
		bcf STATUS,RP0 			; Bank 0
		bcf INTCON,GPIE			; Disables the GPIO port change interrupt
		clrf GPIO					; reset outputs for future
		bsf STATUS,RP0 			; Bank 1
      clrf IOCB					; zakaz vsech INTERRUPT-ON-CHANGE
      clrf WPU						; Pull-up disabled for all inputs
      bsf OPTION_REG,NOT_GPPU	; GPIO pull-ups are disabled
		bcf TRISIO,Q_OUT			; Vystup Q
		bcf TRISIO,NOT_Q_OUT		; Vystup Negace Q
		bcf TRISIO,NC1				; Rezerva
		bcf TRISIO,NC2				; Rezerva
		bcf TRISIO,NC3				; Rezerva
		bcf STATUS, RP0 			; Restore Bank 0
		return


;----------------------------------------------------------------------
;		Timer1 Initialization segment with run and start its interrupt only
;----------------------------------------------------------------------
set_TMR1
		clrf T1CON 					; Stop Timer1,LP oscillator is off,Internal clock (F OSC /4)
		bsf T1CON,T1CKPS1			; prescaler /8
		bsf T1CON,T1CKPS0			; prescaler /8
		bsf STATUS,RP0 			; Bank 1
		bcf PIE1,TMR1IE			; Disables the TMR1 overflow interrupt
		bcf STATUS,RP0 			; Bank 0
		bcf PIR1,TMR1IF 			; clear TMR1 IRQ flag - register overflowed
		clrf TMR1L 					; set Timer1 LO byte register to zero
		clrf TMR1H					; set Timer1 HI byte register to zero
		bsf STATUS,RP0 			; Bank 1
		bsf PIE1,TMR1IE			; Enables the TMR1 overflow interrupt
		bcf STATUS,RP0 			; Bank 0
		bsf T1CON,TMR1ON 			; Timer1 starts to increment
		return

     
 
		END                     ; directive 'end of program'

